#include <os.h>
#include "screen.h"
#include "charmap.h"
#define SCREEN_CONTRAST_ADDR 0x900F0020

#define SCREEN_BASE_PTR		0xC0000010 
#define SCREEN_MODE_ADDR	0xC000001C
#define SCREEN_INT_ADDR		0xC0000020
unsigned short int* SCREEN_BASE_ADDR = 0;
unsigned short int* SCREEN_ORIG_ADDR = 0;
#define SCREEN4_SIZE	SCREEN_WIDTH*SCREEN_HEIGHT/2
#define SCREEN16_SIZE	SCREEN_WIDTH*SCREEN_HEIGHT*2

int mode_bits=4;
int contrast=0;
void initScr()
{	SCREEN_BASE_ADDR	=*(unsigned short int**)SCREEN_BASE_PTR;
	SCREEN_ORIG_ADDR=SCREEN_BASE_ADDR;
	mode_bits=is_cx?16:4;
	contrast=*(volatile unsigned*) SCREEN_CONTRAST_ADDR;
}

void initScr16()
{	if(!is_cx)
	{	int mode = *(volatile unsigned*) SCREEN_MODE_ADDR;
		mode = mode&~0b1110;
		mode = mode|0b1000;
		*(volatile unsigned*) SCREEN_MODE_ADDR = mode;
		mode_bits=16;
	}
}

void endScr16()
{	if(mode_bits==16 && !is_cx)
	{	int mode = *(volatile unsigned*) SCREEN_MODE_ADDR;
		mode = mode&~0b100001110;
		mode = mode|0b0100;
		*(volatile unsigned*) SCREEN_MODE_ADDR = mode;
		mode_bits=4;
		*(volatile unsigned*) SCREEN_BASE_PTR = SCREEN_ORIG_ADDR;
		SCREEN_BASE_ADDR=SCREEN_ORIG_ADDR;
	}
} 



unsigned short int* getScreen()
{	return SCREEN_BASE_ADDR;
}

void setScreen(unsigned short int* buf)
{	SCREEN_BASE_ADDR=buf;
	*(volatile unsigned*) SCREEN_BASE_PTR = buf;
}

unsigned int getContrast()
{ return contrast;//*(volatile unsigned*) SCREEN_CONTRAST_ADDR;
}

int off_mode=0;
unsigned int setContrast(unsigned int level)
{ 	unsigned int lowlevel = level &0b11111111;
	if(lowlevel>CONTRAST_MAX) lowlevel=CONTRAST_MAX;
	if(lowlevel<CONTRAST_MIN) lowlevel=CONTRAST_MIN;
	level = (level & ~0b11111111)|lowlevel;
	contrast=level;
	if(!is_cx || !off_mode)
		*(volatile unsigned*) SCREEN_CONTRAST_ADDR=level;
	return level;
}

void switchScrOffOn(int s)
{
if(!is_cx)
{
	int mask = 0b100000000001;
	int mode = *(volatile unsigned*) SCREEN_MODE_ADDR;
	if(s)	mode |= mask;
	else	mode &= ~mask;
	*(volatile unsigned*) SCREEN_MODE_ADDR = mode;	
//	return mode&mask;
}
else
{	if(s) off_mode=0;
	else off_mode=1;
	if(!off_mode)
		setContrast(contrast);
	else
		*(volatile unsigned*) SCREEN_CONTRAST_ADDR=(contrast&~0b11111111)|CONTRAST_MIN;
}
}

void setBufPixel(unsigned short int* buf, int x, int y, unsigned short int color)
{	if(x >= 0 && x < SCREEN_WIDTH && y >= 0 && y < SCREEN_HEIGHT)
		buf[y*SCREEN_WIDTH+x]=color;
}


void setBufPixelRGB(unsigned short int* buf, int x, int y, unsigned int r, unsigned int g, unsigned int b)
{	if(x >= 0 && x < SCREEN_WIDTH && y >= 0 && y < SCREEN_HEIGHT)
		{	unsigned int color;
			if(is_cx) color=(r<<11)|(g<<5)|b;
			else	  color=(~((r*30+g*59/2+b*11)/100/2))<<1;
			buf[y*SCREEN_WIDTH+x]=color;
		}
}

void putBufChar(unsigned short int* buf, int x, int y, char ch, int trsp)
{
  int i, j, pixelOn;
  for(i = 0; i < CHAR_HEIGHT; i++)  
  {
    for(j = 0; j < CHAR_WIDTH; j++) 
    {
      pixelOn = charMap_ascii[(unsigned char)ch][i] << j ;  
      pixelOn = pixelOn & 0x80 ;  
      if (pixelOn) {
        if(trsp && is_cx)
		setBufPixel(buf,x + j, y + i, 0); 
	else
		setBufPixel(buf,x + j, y + i, 0xFFFF); 
      } else if(!trsp) {
        setBufPixel(buf,x + j, y + i, 0);
      } 
    }
  } 
}

void drwBufStr(unsigned short int* buf, int x, int y, char* str, int ret, int trsp)
{ int l = strlen(str);
  int i;
  int stop=0;
  for (i = 0; i < l && !stop; i++) {
    if (str[i] == 0x0A) {
      if(ret)
      { x = 0;
        y += CHAR_HEIGHT;
      }
      else
      { putBufChar(buf, x,y, ' ',trsp); 
        x += CHAR_WIDTH;
      }
    } else {
      putBufChar(buf, x, y, str[i],trsp);
      x += CHAR_WIDTH;
    }
    if (x >= SCREEN_WIDTH-CHAR_WIDTH)
    { if(ret)
      { x = 0;
        y += CHAR_HEIGHT;
      }
      else
        stop=1;
    }
  }
}

void dispBufImgRGB(unsigned short int* buf, int xoff, int yoff, char* img, int width, int height,int border)
{	int dwidth=width, dheight=height;
	int data_x=0, data_y=0;
	unsigned int x = 0, y = 0;
	float i, j;
	if(xoff < 0){
		dwidth = dwidth + xoff;
		data_x = (int)(-xoff);
		xoff = 0;
	}
	if(yoff < 0){
		dheight = dheight + yoff;
		data_y = (int)(-yoff);	
		yoff = 0;
	}
	int r,g,b;
	for(i=0, x=0; (int)i < dwidth && x < SCREEN_WIDTH; i+= 1, x++)
		for(j=0, y=0; (int)j < dheight && y < SCREEN_HEIGHT; j+= 1, y++)
		{	b=img[(((int)j+data_y)*width+(int)i+data_x)*3];
			g=img[(((int)j+data_y)*width+(int)i+data_x)*3+1];
			r=img[(((int)j+data_y)*width+(int)i+data_x)*3+2];
			if(border && b==0 && g==0 && r==0 && !is_cx)
			{	r=255; g=255; b=255;
			}
			r=r>>3;
			g=g>>2;
			b=b>>3;
			if(!is_cx)
			{	r=31-(31-r);
				g=63-(63-r);
				b=31-(31-b);
			}
			setBufPixelRGB(buf, xoff + x, yoff + dheight-1-y, r, g, b);
		}
}

void dispBufIMG(unsigned short int* buf, int xoff, int yoff, unsigned short int* img, int width, int height, float inc)
{	int dwidth=width, dheight=height;
	int data_x=0, data_y=0;
	unsigned int x,y;
	dwidth = (int)(dwidth/inc);
	dheight = (int)(dheight/inc);
	if(xoff < 0){
		dwidth = dwidth + xoff;
		data_x = (int)(-xoff*inc);
		xoff = 0;
	}
	if(yoff < 0){
		dheight = dheight + yoff;
		data_y = (int)(-yoff*inc);	
		yoff = 0;
	}
	dwidth = min(dwidth,SCREEN_WIDTH);
	dheight = min(dheight,SCREEN_HEIGHT);
	for(x=xoff; x < dwidth; x++)
		for(y=yoff; y < dheight; y++)
			setBufPixel(buf, x, y, img[((((int)(y*inc))+data_y)*width+((int)(x*inc))+data_x)]);
}

void clrBuf(unsigned short int* buf)
{	int color=0;
	memset(buf,color,SCREEN16_SIZE);
}

void clrScr()
{	clrBuf(SCREEN_BASE_ADDR);
}

void clrBufBox(unsigned short int* buf, int x,int y, int w, int h)
{	int i,j;
	if(x<0) { w=w+x; x=0; }
	if(y<0) { h=h+y; y=0; }
	if(y+h>SCREEN_HEIGHT) h=SCREEN_HEIGHT-y;
	if(x+w>SCREEN_WIDTH) w=SCREEN_WIDTH-x;
	int color=0;
	for(j = y; j < y+h; j ++)
	{	i=x;
		memset(buf+(j*SCREEN_WIDTH+i),color,w*2);
	}
}

void refreshScr(unsigned char* buf)
{	memcpy(SCREEN_BASE_ADDR,buf,SCREEN_WIDTH*SCREEN_HEIGHT*mode_bits/8);
}

void drwBufHoriz(unsigned short int* buf, int y, int x1, int x2, int color)
{	int m = max(x1,x2);
	int i = min(x1,x2);
	while(i<=m)
	{	setBufPixel(buf,i,y,color);
		i++;
	}
}

void drwBufFullHoriz(unsigned short int* buf, int y)
{	drwBufHoriz(buf, y, 0, SCREEN_WIDTH-1, 0x00);
}

void drwBufVert(unsigned short int* buf, int x, int y1, int y2, int color)
{	int m = max(y1,y2);
	int i = min(y1,y2);
	while(i<=m)
	{	setBufPixel(buf, x,i,color);
		i++;
	}
}

void drwBufBox(unsigned short int* buf, int x1, int y1, int x2, int y2, int color)
{	drwBufHoriz(buf, y1,x1,x2,color);
	drwBufHoriz(buf, y2,x1,x2,color);
	drwBufVert(buf, x1,y1,y2,color);
	drwBufVert(buf, x2,y1,y2,color);
}